/* Copyright aQute SARL, All Rights Reserved
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 */
package aQute.fileinstall;

import java.util.*;

import org.osgi.framework.*;
import org.osgi.service.cm.*;
import org.osgi.service.packageadmin.*;
import org.osgi.util.tracker.*;

/**
 * This clever little bundle watches a directory and will install any jar file
 * if finds in that directory (as long as it is a valid bundle and not a
 * fragment).
 * 
 */

public class FileInstall implements BundleActivator, ManagedServiceFactory {
	static ServiceTracker		padmin;
	static ServiceTracker		cmTracker;

	BundleContext				context;
	Map							watchers			= new HashMap();
	

	public void start(BundleContext context) throws Exception {
		System.out.println("FILEINSTALL | aQute (c) 2007 v1.3");
		this.context = context;
		Hashtable props = new Hashtable();
		props.put(Constants.SERVICE_PID, getName() );
		context.registerService(ManagedServiceFactory.class.getName(),this,props);
		
		padmin = new ServiceTracker(context, PackageAdmin.class.getName(), null);
		padmin.open();
		cmTracker = new ServiceTracker(context, ConfigurationAdmin.class
				.getName(), null);
		cmTracker.open();
		
		// Created the initial configuration
		Hashtable ht= new Hashtable();
		set(ht, DirectoryWatcher.POLL);
		set(ht, DirectoryWatcher.DIR);
		set(ht, DirectoryWatcher.DEBUG);
		updated("initial", ht );
	}

	private void set(Hashtable ht, String key) {
		Object o = context.getProperty(key);
		if ( o == null )
			return;
		ht.put(key,o);
	}

	public void stop(BundleContext context) throws Exception {
		for(Iterator w = watchers.values().iterator(); w.hasNext(); ) try {
			DirectoryWatcher dir = (DirectoryWatcher) w.next();
			w.remove();
			dir.close();
		} catch( Exception e) {
			// Ignore
		}
		cmTracker.close();
		padmin.close();
	}

	public void deleted(String pid) {
		DirectoryWatcher watcher = (DirectoryWatcher) watchers.remove(pid);
		if ( watcher != null )
			watcher.close();
	}

	public String getName() {
		return "biz.aQute.fileInstall";
	}

	public void updated(String pid, Dictionary properties)
			throws ConfigurationException {
		deleted(pid);
		DirectoryWatcher watcher = new DirectoryWatcher(properties, context);
		watchers.put(pid, watcher);
		watcher.start();
	}
}
